import {
  Modal,
  ModalService,
  FileInput,
  SelectMultiple,
  LoadingService,
  Radio,
  TextInput,
} from '@zeedhi/common';
import {
  Metadata,
  Http,
  I18n,
} from '@zeedhi/core';
import {
  read,
  utils,
  writeFile,
  WorkBook,
} from 'xlsx';
import { Utilities } from '@/util/Utilities';

export class ImportAndExportSpreadsheetController {
  private importAndExportSheetModal!: Modal;
  
  public importSpreadsheet = true;
  
  public exportSpreadsheet = false;
  
  public titleModal = '';
  
  public async createImportAndExportSheetModal() {
    this.importAndExportSheetModal = await ModalService.createFromJson('/modals/importAndExportSpreadsheetModal', true);
    this.importAndExportSheetModal.show();
  }
  
  public destroyImportAndExportSheetModal() {
    this.importAndExportSheetModal?.destroy();
  }
  
  public async openImportAndExportSheetModal({ component }) {
    this.importSpreadsheet = component.name === 'spreadsheetImportButton';
    if (this.importSpreadsheet) {
      this.titleModal = 'Importação de Planilha';
      this.exportSpreadsheet = false;
    } else {
      this.titleModal = 'Download de Planilha';
      this.exportSpreadsheet = true;
    }
    
    if (!this.importAndExportSheetModal) {
      await this.createImportAndExportSheetModal();
    } else {
      await this.importAndExportSheetModal.show();
    }
  }
  
  public closeImportAndExportSheetModal() {
    this.importAndExportSheetModal.hide();
  }
  
  public importSheet() {
    const utilities = new Utilities();
    const fileInput: FileInput = Metadata.getInstance('fileInputImport');
    const routeName = 'importBudget';
    const file = fileInput.value[0];
    const reader = new FileReader();
  
    reader.onload = async (event) => {
      const data = new Uint8Array(event.target.result as ArrayBuffer);
      const workbook = read(data, { type: 'array' });
  
      const sheetName = workbook.SheetNames[0];
      const sheet = workbook.Sheets[sheetName];
      const jsonData = utils.sheet_to_json(sheet);
      
      LoadingService.show();
      
      await Http.post(`/${routeName}`, {
        requestType: 'Row',
        row: jsonData,
      }).then((response) => {
        if (response) {
          console.warn('Dados enviados com sucesso');
        } else {
          console.error('Erro ao enviar dados');
        }
        
        LoadingService.hide();
      }).catch((e) => {
        utilities.errorMessage('Falha ao buscar dados');
        LoadingService.hide();
        console.warn(e);
      });
    };
  
    reader.readAsArrayBuffer(file);
  }
  
  public async exportSheet() {
    const utilities = new Utilities();
    const routeName = 'exportBudget';
    const selectUnit = Metadata.getInstance<SelectMultiple>('unitExportFile');
    const selectRadio = Metadata.getInstance<Radio>('radioExportFile');
    const year = Metadata.getInstance<TextInput>('nrYear');
    const startValue = Metadata.getInstance<TextInput>('checkStartValue');
    let data;
    
    const descriptionMap: { [key: number]: string } = {
      0: 'Faturamento',
      1: 'Imposto',
      2: 'Consumo',
      3: 'Mão de Obra',
      4: 'Taxa Administrativa',
      5: 'Despesas da Matriz',
      6: 'Despesas Diversas',
      7: 'Despesas Lanc. Entrada',
      8: 'Imposto a Recuperar',
      9: 'Impostos Sobre o Lucro',
      10: 'Investimentos',
      11: 'Depreciação',
    };
    
    if (selectUnit.value.length > 0 && year.value.length > 0) {
      const sendObject = {
        CDFILIAL: selectUnit.value,
        MODELO: selectRadio.value,
        NRANO: year.value,
        STARTVALUE: startValue.value,
      };
      
      LoadingService.show();
      await Http.post(`/${routeName}`, sendObject).then((response) => {
        data = response.data.dataset[routeName];
        
        data.forEach((obj) => {
          if (typeof obj.CDORCAMENTO === 'number') {
            obj.NMORCAMENTO = I18n.translate('gop::'.concat(descriptionMap[obj.CDORCAMENTO]));
          } else {
            Object.entries(obj).forEach(([key, value]) => {
              obj[key] = I18n.translate('gop::'.concat(value as string));
            });
          }
        });
        LoadingService.hide();
        
        this.generateSpreadsheet(data, 'xlsx', 'GOP10000');
      }).catch((e) => {
        utilities.errorMessage('Falha ao buscar dados');
        LoadingService.hide();
        console.warn(e);
      });
    } else {
      selectUnit.validate();
      year.validate();
    }
  }
  
  generateSpreadsheet(data: any[], fileType: 'xls' | 'xlsx' | 'ods', fileName: string) {
    const headers = [];
    const worksheet = utils.json_to_sheet(data, { header: headers, skipHeader: true });
    
    utils.sheet_add_aoa(worksheet, [headers], { origin: 'A1' });
  
    worksheet['!rows'] = [{ hidden: true }];
  
    const workbook: WorkBook = { Sheets: { Sheet1: worksheet }, SheetNames: ['Sheet1'] };

    writeFile(workbook, `${fileName}.${fileType}`);
  }
}
